@@ -17,6 +17,7 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 use std::path::{Path, PathBuf};
+use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
 
 use nucliadb_core::prelude::*;
 use nucliadb_core::protos::shard_created::{
@@ -34,11 +35,15 @@ use crate::shards::metadata::ShardMetadata;
 use crate::shards::versions::Versions;
 use crate::telemetry::run_with_telemetry;
 
+const MERGE_THRESHOLD: usize = 5;
+
 #[derive(Debug)]
 pub struct ShardWriter {
     pub metadata: ShardMetadata,
     pub id: String,
     pub path: PathBuf,
+    merging_available: AtomicBool,
+    writes_without_merge: AtomicUsize,
     text_writer: TextsWriterPointer,
     paragraph_writer: ParagraphsWriterPointer,
     vector_writer: VectorsWriterPointer,
@@ -49,6 +54,28 @@ pub struct ShardWriter {
     relation_service_version: i32,
 }
 
+impl PartialOrd for ShardWriter {
+    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
+        self.id.partial_cmp(&other.id)
+    }
+}
+impl PartialEq for ShardWriter {
+    fn eq(&self, other: &Self) -> bool {
+        self.id.eq(&other.id)
+    }
+}
+impl Ord for ShardWriter {
+    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
+        self.id.cmp(&other.id)
+    }
+}
+impl Eq for ShardWriter {}
+impl std::hash::Hash for ShardWriter {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.id.hash(state)
+    }
+}
+
 impl ShardWriter {
     #[tracing::instrument(skip_all)]
     fn initialize(
@@ -100,6 +127,8 @@ impl ShardWriter {
             paragraph_writer: paragraphs.unwrap(),
             vector_writer: vectors.unwrap(),
             relation_writer: relations.unwrap(),
+            merging_available: AtomicBool::new(true),
+            writes_without_merge: AtomicUsize::new(0),
             document_service_version: versions.version_texts() as i32,
             paragraph_service_version: versions.version_paragraphs() as i32,
             vector_service_version: versions.version_vectors() as i32,
@@ -290,6 +319,7 @@ impl ShardWriter {
         paragraph_result?;
         vector_result?;
         relation_result?;
+        self.writes_without_merge.fetch_add(1, Ordering::SeqCst);
         Ok(())
     }
 
@@ -347,6 +377,7 @@ impl ShardWriter {
         paragraph_result?;
         vector_result?;
         relation_result?;
+        self.writes_without_merge.fetch_add(1, Ordering::SeqCst);
         Ok(())
     }
 
@@ -447,8 +478,46 @@ impl ShardWriter {
         text_read(&self.text_writer).count()
     }
 
+    #[tracing::instrument(skip_all)]
+    pub fn merge(&self) -> NodeResult<()> {
+        if !self.merging_available.swap(false, Ordering::SeqCst) {
+            return Ok(());
+        }
+
+        // Critical section, only one merge at a time
+
+        let current_writes = self.writes_without_merge.swap(0, Ordering::SeqCst);
+        let merge_result = if current_writes >= MERGE_THRESHOLD {
+            self.inner_merge()
+        } else {
+            self.writes_without_merge
+                .fetch_add(current_writes, Ordering::SeqCst);
+            Ok(())
+        };
+
+        // Merging is available again
+        self.merging_available.store(true, Ordering::SeqCst);
+        merge_result
+    }
+
+    fn inner_merge(&self) -> NodeResult<()> {
+        // TODO: add relations merge when it becomes a Tantivy only index.
+        paragraph_write(&self.paragraph_writer).merge()?;
+        text_write(&self.text_writer).merge()?;
+        vector_write(&self.vector_writer).merge()?;
+
+        // After merging GC is applied
+        // TODO: add relations gc when it becomes a Tantivy only index.
+        paragraph_write(&self.paragraph_writer).garbage_collection()?;
+        text_write(&self.text_writer).garbage_collection()?;
+        vector_write(&self.vector_writer).garbage_collection()?;
+
+        Ok(())
+    }
+
     #[tracing::instrument(skip_all)]
     pub fn gc(&self) -> NodeResult<()> {
-        vector_write(&self.vector_writer).garbage_collection()
+        // TODO: remove this operation from the interface
+        Ok(())
     }
 }
