@@ -405,6 +405,36 @@ pub struct FieldMetadata {
     pub relations: ::prost::alloc::vec::Vec<Relations>,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
+pub struct QuestionAnswer {
+    #[prost(string, tag="1")]
+    pub question: ::prost::alloc::string::String,
+    #[prost(string, tag="2")]
+    pub question_language: ::prost::alloc::string::String,
+    #[prost(string, tag="3")]
+    pub answer: ::prost::alloc::string::String,
+    #[prost(string, tag="4")]
+    pub answer_language: ::prost::alloc::string::String,
+    #[prost(int64, repeated, tag="5")]
+    pub paragraph_ids: ::prost::alloc::vec::Vec<i64>,
+}
+#[derive(Clone, PartialEq, ::prost::Message)]
+pub struct FieldQuestionAnswerWrapper {
+    #[prost(message, optional, tag="3")]
+    pub field: ::core::option::Option<FieldId>,
+    #[prost(oneof="field_question_answer_wrapper::FileOrData", tags="1, 2")]
+    pub file_or_data: ::core::option::Option<field_question_answer_wrapper::FileOrData>,
+}
+/// Nested message and enum types in `FieldQuestionAnswerWrapper`.
+pub mod field_question_answer_wrapper {
+    #[derive(Clone, PartialEq, ::prost::Oneof)]
+    pub enum FileOrData {
+        #[prost(message, tag="1")]
+        QuestionAnswer(super::QuestionAnswer),
+        #[prost(message, tag="2")]
+        File(super::CloudFile),
+    }
+}
+#[derive(Clone, PartialEq, ::prost::Message)]
 pub struct FieldComputedMetadata {
     #[prost(message, optional, tag="1")]
     pub metadata: ::core::option::Option<FieldMetadata>,
@@ -784,7 +814,7 @@ pub struct FieldId {
     #[prost(string, tag="2")]
     pub field: ::prost::alloc::string::String,
 }
-/// This message is used to store a list of all field ids of a particular 
+/// This message is used to store a list of all field ids of a particular
 /// resource. Note that title and summary fields are not included.
 #[derive(Clone, PartialEq, ::prost::Message)]
 pub struct AllFieldIDs {
