@@ -321,14 +321,16 @@ impl WriterChild for VectorWriterService {
     #[measure(actor = "vectors", metric = "garbage_collection")]
     #[tracing::instrument(skip_all)]
     fn garbage_collection(&mut self) -> NodeResult<()> {
-        let time = SystemTime::now();
-
-        let lock = self.index.try_elock()?;
+        let lock = self.index.get_elock()?;
         self.index.collect_garbage(&lock)?;
+        Ok(())
+    }
 
-        let took = time.elapsed().map(|i| i.as_secs_f64()).unwrap_or(f64::NAN);
-        debug!("Garbage collection {took} ms");
-
+    #[measure(actor = "vectors", metric = "merge")]
+    #[tracing::instrument(skip_all)]
+    fn merge(&mut self) -> NodeResult<()> {
+        let lock = self.index.get_slock()?;
+        self.index.merge(&lock)?;
         Ok(())
     }
 }
