@@ -33,7 +33,7 @@ use pyo3::prelude::*;
 use pyo3::types::PyList;
 use pyo3::PyErr;
 
-use crate::errors::{IndexNodeException, LoadShardError, ShardNotFound};
+use crate::errors::{IndexNodeException, LoadShardError};
 use crate::RawProtos;
 
 #[pyclass]
@@ -72,19 +72,14 @@ pub struct NodeReader {
 
 impl NodeReader {
     fn obtain_shard(&self, shard_id: String) -> Result<Arc<ShardReader>, PyErr> {
-        let loaded = self.shards.load(shard_id.clone());
-        if let Err(error) = loaded {
-            return Err(LoadShardError::new_err(format!(
+        if let Some(shard) = self.shards.get(shard_id.clone()) {
+            return Ok(shard);
+        }
+        match self.shards.load(shard_id.clone()) {
+            Ok(shard) => Ok(shard),
+            Err(error) => Err(LoadShardError::new_err(format!(
                 "Error loading shard {}: {}",
                 shard_id, error
-            )));
-        }
-
-        match self.shards.get(shard_id.clone()) {
-            Some(shard) => Ok(shard),
-            None => Err(ShardNotFound::new_err(format!(
-                "Shard {} not found",
-                shard_id
             ))),
         }
     }
